/*
  fr-FR.h - localization for French - France for Tasmota

  Copyright (C) 2021  Olivier Francais

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _LANGUAGE_FR_FR_H_
#define _LANGUAGE_FR_FR_H_

/*************************** ATTENTION *******************************\
 *
 * Due to memory constraints only UTF-8 is supported.
 * To save code space keep text as short as possible.
 * Time and Date provided by SDK can not be localized (yet).
 * Use online command StateText to translate OFF, ON, TOGGLE and HOLD.
 * Use online command Prefix to translate cmnd, stat and tele.
 *
 * Updated until v12.4.0
\*********************************************************************/

#define LANGUAGE_MODULE_NAME         // Enable to display "Module Generic" (ie Spanish), Disable to display "Generic Module" (ie English)

#define LANGUAGE_LCID 1036
// HTML (ISO 639-1) Language Code
#define D_HTML_LANGUAGE "fr"

// "2017-03-07T11:08:02" - ISO8601:2004
#define D_YEAR_MONTH_SEPARATOR "-"
#define D_MONTH_DAY_SEPARATOR "-"
#define D_DATE_TIME_SEPARATOR "T"
#define D_HOUR_MINUTE_SEPARATOR ":"
#define D_MINUTE_SECOND_SEPARATOR ":"

#define D_DAY3LIST "DimLunMarMerJeuVenSam"
#define D_MONTH3LIST "JanFevMarAvrMaiJunJulAutSepOctNovDec"

// Non JSON decimal separator
#define D_DECIMAL_SEPARATOR ","

// Common
#define D_ABSOLUTE_HUMIDITY "Abs Humidity"
#define D_ADMIN "Admin"
#define D_AIR_QUALITY "Qualité de l'Air"
#define D_AP "AP"                    // Access Point
#define D_AS "comme"
#define D_AUTO "AUTO"
#define D_BATT "Batt"                // Short for Battery
#define D_BATTERY_CHARGE "Charge"    // Battery charge in %
#define D_BLINK "Blink"              // Not better in french
#define D_BLINKOFF "BlinkOff"
#define D_BOOT_COUNT "Nb. de boot"
#define D_BRIGHTLIGHT "Luminosité"
#define D_BSSID "BSSId"
#define D_BUTTON "Bouton"
#define D_BY "par"                    // Written by me
#define D_BYTES "Octets"
#define D_CELSIUS "Celsius"
#define D_CHANNEL "Canal"
#define D_CO2 "Dioxyde de carbone"
#define D_CODE "code"                // Button code
#define D_COLDLIGHT "Froid"
#define D_COMMAND "Commande"
#define D_CONNECTED "Connecté"
#define D_CORS_DOMAIN "Domaine CORS"
#define D_COUNT "Compte"
#define D_COUNTER "Compteur"
#define D_CT_POWER "Puissance CT"
#define D_CURRENT "Courant"          // As in Voltage and Current
#define D_CURRENT_NEUTRAL "Courant neutre"
#define D_DATA "Donnée"
#define D_DARKLIGHT "Sombre"
#define D_DEBUG "Debug"
#define D_DEWPOINT "Point de rosée"
#define D_DISABLED "Désactivé"
#define D_MOVING_DISTANCE "Distance mobile"
#define D_STATIC_DISTANCE "Distance fixe"
#define D_DETECT_DISTANCE "Distance détectée"
#define D_DISTANCE "Distance"
#define D_DNS_SERVER "Serveur DNS"
#define D_DO "Oxygène dissout"
#define D_DONE "Terminé"
#define D_DST_TIME "DST"             // Daylight Saving Time
#define D_EC "EC"
#define D_ECO2 "eCO₂"
#define D_EMULATION "Émulation"
#define D_ENABLED "Activé"
#define D_ERASE "Effacer"
#define D_ERROR "Erreur"
#define D_FAHRENHEIT "Fahrenheit"
#define D_FAILED "Échoué"
#define D_FALLBACK "Secours"
#define D_FALLBACK_TOPIC "Topic de secours"
#define D_FALSE "Faux"
#define D_FILE "Fichier"
#define D_FILE_SYSTEM_SIZE "File System Size"
#define D_FLOW_RATE "Débit"
#define D_FRAGMENTATION "frag."      // Lower case abbreviated version of fragmentation used in "memory fragmentation"
#define D_FRAME_RATE "Fréq. image"
#define D_FREE_MEMORY "Mémoire libre"
#define D_PSR_MAX_MEMORY "Mémoire PS-RAM"
#define D_PSR_FREE_MEMORY "Mémoire PS-RAM libre"
#define D_FREQUENCY "Fréquence"
#define D_GAS "Gaz"
#define D_GATEWAY "Passerelle"
#define D_GROUP "Groupe"
#define D_HOST "Hôte"
#define D_HALL_EFFECT "Effet Hall"
#define D_HOSTNAME "Nom d'Hôte"
#define D_HUMIDITY "Humidité"
#define D_ILLUMINANCE "Éclairement"
#define D_IMMEDIATE "immédiat"       // Button immediate
#define D_INDEX "Index"
#define D_INFO "Info"
#define D_INFRARED "Infra-rouge"
#define D_INITIALIZED "Initialisé"
#define D_IP_ADDRESS "Adresse IP"
#define D_LIGHT "Lumière"
#define D_LWT "LWT"					         // MQTT Last Will and Testament
#define D_LQI "LQI"                  // ZigBee Link Quality Index
#define D_MODULE "Module"
#define D_MOISTURE "Humidité"
#define D_MQTT "MQTT"
#define D_MULTI_PRESS "multi-pression"
#define D_NOISE "Bruit"
#define D_NONE "Aucun"
#define D_NOX "NOx"
#define D_O2 "Oxygène"
#define D_OFF "Arrêt"
#define D_OFFLINE "Déconnecté"
#define D_OK "Ok"
#define D_ON "Marche"
#define D_ONLINE "Connecté"
#define D_ORP "Redox"                // Oxidation Reduction Potential
#define D_PASSWORD "Mot de passe"
#define D_PH "pH"
#define D_MQ "MQ"                    // MQ series Gas sensor
#define D_PARTITION "Partition"      // As in flash and firmware partition
#define D_PORT "Port"
#define D_POWER_FACTOR "Fact de puiss"
#define D_POWERUSAGE "Puissance"
#define D_POWERUSAGE_ACTIVE "Puiss active"
#define D_POWERUSAGE_ACTIVE_TOTAL "Active Power Total"
#define D_POWERUSAGE_APPARENT "Puiss apparente"
#define D_POWERUSAGE_REACTIVE "Puiss réactive"
#define D_PRESSURE "Pression"
#define D_PRESSUREATSEALEVEL "PressionMer"
#define D_PROGRAM_FLASH_SIZE "Taille Flash Programme"
#define D_PROGRAM_SIZE "Taille programme"
#define D_PROJECT "Projet"
#define D_RAIN "Pluie"
#define D_RANGE "Intervalle"
#define D_RECEIVED "Reçu"
#define D_RESTART "Redémarrage"
#define D_RESTARTING "Redémarre"
#define D_RESTART_REASON "Raison du redémarrage"
#define D_RESTORE "restaurer"
#define D_RETAINED "persistant"		   // MQTT
#define D_RULE "Règle"
#define D_SAVE "Enregistrer"
#define D_SENSOR "Capteur"
#define D_SSID "SSID"
#define D_START "Lancer"	         // "Lancer la mise à jour"
#define D_STD_TIME "STD"
#define D_STOP "Stop"
#define D_SUBNET_MASK "Masque sous-réseau"
#define D_SUBSCRIBE_TO "S'abonner à"
#define D_UNSUBSCRIBE_FROM "Se désabonner de"
#define D_SUCCESSFUL "Réussi"
#define D_SUNRISE "Lever du jour"
#define D_SUNSET "Tombée de la nuit"
#define D_TEMPERATURE "Température"
#define D_TO "à"
#define D_TOGGLE "Inverser"
#define D_TOPIC "Topic"              // Keep MQTT keyword
#define D_TOTAL_USAGE "Eau totale"
#define D_TRANSMIT "Transmettre"
#define D_TRUE "Vrai"
#define D_TVOC "TVOC"				         // not used ???
#define D_TWILIGHT_ASTRONOMICAL "Astronomical"
#define D_TWILIGHT_CIVIL "Civil"
#define D_TWILIGHT_NAUTICAL "Nautical"
#define D_UPGRADE "la mise à jour" 	 // "Lancer la mise à jour"
#define D_UPLOAD "Upload"            // Not better in french
#define D_UPTIME "Durée d'activité"
#define D_USED "utilisé"
#define D_USER "Utilisateur"
#define D_UTC_TIME "UTC"             // Coordinated Universal Time
#define D_UV_INDEX "Indice UV"
#define D_UV_INDEX_1 "Faible"
#define D_UV_INDEX_2 "Modéré"
#define D_UV_INDEX_3 "Élevé"
#define D_UV_INDEX_4 "Très élevé"
#define D_UV_INDEX_5 "Brûlure niv.1/2"
#define D_UV_INDEX_6 "Brûlure niv.3"
#define D_UV_INDEX_7 "Hors échelle"
#define D_UV_LEVEL "Niveau UV"
#define D_UV_POWER "Puissance UV"
#define D_VERSION "Version"
#define D_VOC "VOC"
#define D_VOLTAGE "Tension"
#define D_VOLUME "Volume"
#define D_WEIGHT "Poids"
#define D_WARMLIGHT "Chaud"
#define D_WEB_SERVER "Serveur web"
#define D_SOC "État de la Charge"
#define D_SOH "État de Santé"

// tasmota.ino
#define D_WARNING_MINIMAL_VERSION "ATTENTION Cette version ne gère pas les réglages persistants"
#define D_LEVEL_10 "niveau 1-0"
#define D_LEVEL_01 "niveau 0-1"
#define D_SERIAL_LOGGING_DISABLED "Journalisation série désactivée"
#define D_SYSLOG_LOGGING_REENABLED "Jounalisation SysLog réactivée"

#define D_SET_BAUDRATE_TO "Définir le débit à"
#define D_RECEIVED_TOPIC "Topic reçu" // Terme MQTT
#define D_DATA_SIZE "Taille données"
#define D_ANALOG_INPUT "Analogique"

// support.ino
#define D_OSWATCH "osWatch"          // Indicateurs 
#define D_BLOCKED_LOOP "Boucle bloquée"
#define D_WPS_FAILED_WITH_STATUS "WPSconfig ÉCHOUÉ avec status"
#define D_ACTIVE_FOR_3_MINUTES "actif pour 3 minutes"
#define D_FAILED_TO_START "Échec du démarrage"
#define D_PATCH_ISSUE_2186 "Correctif 2186"
#define D_CONNECTING_TO_AP "Connexion à l'AP"
#define D_IN_MODE "en mode"
#define D_CONNECT_FAILED_NO_IP_ADDRESS "Échec de connexion car aucune adresse IP n'a été reçue"
#define D_CONNECT_FAILED_AP_NOT_REACHED "Échec de connexion car l'AP ne peut-être contacté"
#define D_CONNECT_FAILED_WRONG_PASSWORD "Échec de connexion"
#define D_CONNECT_FAILED_AP_TIMEOUT "Échec de connexion avec l'AP, expiré"
#define D_ATTEMPTING_CONNECTION "Tentative de connexion..."
#define D_CHECKING_CONNECTION "Vérification connexion..."
#define D_QUERY_DONE "Requête terminée. Services MQTT trouvés"
#define D_MQTT_SERVICE_FOUND "Service MQTT trouvé sur"
#define D_FOUND_AT "trouvé à"
#define D_SYSLOG_HOST_NOT_FOUND "Hôte SysLog introuvable"

// settings.ino
#define D_SAVED_TO_FLASH_AT "Enregistré en flash à"
#define D_LOADED_FROM_FLASH_AT "Chargé de la flash à"
#define D_USE_DEFAULTS "Utiliser par défaut"
#define D_ERASED_SECTOR "Secteur effacé"

// xdrv_02_webserver.ino
#define D_NOSCRIPT "Pour utiliser Tasmota, veuillez activer JavaScript"
#define D_SAFEBOOT "SAFEBOOT"
#define D_MINIMAL_FIRMWARE_PLEASE_UPGRADE "Firmware MINIMAL<br>merci de mettre à jour"
#define D_WEBSERVER_ACTIVE_ON "Serveur web actif sur"
#define D_WITH_IP_ADDRESS "avec l'adresse IP"
#define D_WEBSERVER_STOPPED "Serveur web éteint"
#define D_FILE_NOT_FOUND "Fichier introuvable"
#define D_REDIRECTED "Redirection sur le portail captif"
#define D_WIFIMANAGER_SET_ACCESSPOINT_AND_STATION "Wifimanager en mode Point d'Accès et Station"
#define D_WIFIMANAGER_SET_ACCESSPOINT "Wifimanager en mode Point d'Accès"
#define D_TRYING_TO_CONNECT "Tentative de connexion du module au réseau"

#define D_RESTART_IN "Redémarrage dans"
#define D_SECONDS "secondes"
#define D_DEVICE_WILL_RESTART "Le module va redémarrer dans quelques secondes"
#define D_BUTTON_TOGGLE "on/off"     // Not better in french
#define D_CONFIGURATION "Configuration"
#define D_INFORMATION "Informations"
#define D_FIRMWARE_UPGRADE "Mise à jour du Firmware"
#define D_MANAGEMENT "Consoles"
#define D_CONSOLE "Console"
#define D_CONFIRM_RESTART "Confirmer le redémarrage"

#define D_CONFIGURE_MODULE "Configuration du Module"
#define D_CONFIGURE_WIFI "Configuration WiFi"
#define D_CONFIGURE_MQTT "Configuration MQTT"
#define D_CONFIGURE_DOMOTICZ "Configuration Domoticz"
#define D_CONFIGURE_LOGGING "Configuration du journal"
#define D_CONFIGURE_OTHER "Autre configuration"
#define D_CONFIRM_RESET_CONFIGURATION "Confirmer la réinit. de la config."
#define D_RESET_CONFIGURATION "Configuration par défaut"
#define D_BACKUP_CONFIGURATION "Sauvegarde de la config."
#define D_RESTORE_CONFIGURATION "Restauration de la config."
#define D_MAIN_MENU "Menu principal"

#define D_MODULE_PARAMETERS "Paramètres module"
#define D_MODULE_TYPE "Type de module"
#define D_PULLUP_ENABLE "Bouton/Inter. sans pull-up"
#define D_ADC "ADC"                  // Analog Input pin
#define D_GPIO "GPIO"
#define D_SERIAL_IN "Entrée série"
#define D_SERIAL_OUT "Sortie série"

#define D_WIFI_PARAMETERS "Paramètres Wifi"
#define D_SCAN_FOR_WIFI_NETWORKS "Scan des réseaux wifi"
#define D_SCAN_DONE "Scan terminé"
#define D_NO_NETWORKS_FOUND "Aucun réseau trouvé"
#define D_REFRESH_TO_SCAN_AGAIN "Rafraîchir pour scanner à nouveau"
#define D_DUPLICATE_ACCESSPOINT "Point d'Accès dupliqué"
#define D_SKIPPING_LOW_QUALITY "Évité car de mauvaise qualité"
#define D_MODE "Mode"
#define D_RSSI "RSSI"
#define D_WEP "WEP"
#define D_WPA_PSK "WPA PSK"
#define D_WPA2_PSK "WPA2 PSK"
#define D_AP1_SSID "Réseau WiFi"
#define D_AP1_SSID_HELP "Saisissez ou Choisissez votre réseau WiFi"
#define D_AP2_SSID "Réseau WiFi 2"
#define D_AP2_SSID_HELP "Entrez un réseau WiFi de secours"
#define D_AP_PASSWORD "Mot de passe"
#define D_AP_PASSWORD_HELP "Entrez votre mot de passe WiFi"
#define D_SELECT_YOUR_WIFI_NETWORK "Choisissez votre réseau WiFi"
#define D_SHOW_MORE_WIFI_NETWORKS "Scanner les réseaux WiFi"
#define D_SHOW_MORE_OPTIONS "Plus d'Options"
#define D_CHECK_CREDENTIALS "Veuillez vérifier vos identifiants"
#define D_SUCCESSFUL_WIFI_CONNECTION "Connexion WiFi réussie"
#define D_NOW_YOU_CAN_CLOSE_THIS_WINDOW "Vous pouvez fermer cette fenêtre"
#define D_REDIRECTING_TO_NEW_IP "Redirection vers l'adresse IP d'un autre appareil"

#define D_MQTT_PARAMETERS "Paramètres MQTT"
#define D_CLIENT "Client"
#define D_FULL_TOPIC "Topic complet"

#define D_LOGGING_PARAMETERS "Paramètres du journal"
#define D_SERIAL_LOG_LEVEL "Niveau de journalisation série"
#define D_MQTT_LOG_LEVEL "Niveau de journalisation MQTT"
#define D_WEB_LOG_LEVEL "Niveau de journalisation web"
#define D_SYS_LOG_LEVEL "Niveau SysLog"
#define D_MORE_DEBUG "Plus de debug"
#define D_SYSLOG_HOST "Hôte SysLog"
#define D_SYSLOG_PORT "Port SysLog"
#define D_TELEMETRY_PERIOD "Période télémétrie"

#define D_OTHER_PARAMETERS "Autres paramètres"
#define D_TEMPLATE "Modèle"
#define D_ACTIVATE "Activer"
#define D_DEVICE_NAME "Nom de l'appareil"
#define D_WEB_ADMIN_PASSWORD "Mot de passe Web Admin"
#define D_MQTT_ENABLE "MQTT activé"
#define D_MQTT_TLS_ENABLE "MQTT TLS"
#define D_HTTP_API "HTTP API"
#define D_HTTP_API_ENABLE "HTTP API activé"
#define D_FRIENDLY_NAME "Surnom"
#define D_BELKIN_WEMO "Belkin WeMo"
#define D_HUE_BRIDGE "Hue Bridge"
#define D_SINGLE_DEVICE "module unique"
#define D_MULTI_DEVICE "multi module"

#define D_CONFIGURE_TEMPLATE "Configuration du modèle"
#define D_TEMPLATE_PARAMETERS "Paramètres du modèle"
#define D_TEMPLATE_NAME "Nom"
#define D_BASE_TYPE "Basé sur"
#define D_TEMPLATE_FLAGS "Options"

#define D_SAVE_CONFIGURATION "Enregistrer la configuration"
#define D_CONFIGURATION_SAVED "Configuration enregistrée"
#define D_CONFIGURATION_RESET "Configuration réinitialisée"

#define D_PROGRAM_VERSION "Version du programme"
#define D_BUILD_DATE_AND_TIME "Date & Heure de build"
#define D_CORE_AND_SDK_VERSION "Version Core/SDK"
#define D_FLASH_WRITE_COUNT "Compteur écriture flash"
#define D_MAC_ADDRESS "Adresse MAC"
#define D_MQTT_HOST "Hôte MQTT"
#define D_MQTT_PORT "Port MQTT"
#define D_MQTT_CLIENT "Client MQTT"
#define D_MQTT_USER "Utilisateur MQTT"
#define D_MQTT_TOPIC "Topic MQTT"
#define D_MQTT_GROUP_TOPIC "Groupe topic MQTT"
#define D_MQTT_FULL_TOPIC "Topic complet MQTT"
#define D_MQTT_NO_RETAIN "MQTT No Retain" // Keep MQTT keyword
#define D_MDNS_DISCOVERY "Découverte mDNS"
#define D_MDNS_ADVERTISE "Annonce mDNS"
#define D_ESP_CHIP_ID "ESP Chip Id"  // Not better in french
#define D_FLASH_CHIP_ID "Flash Chip Id"
#define D_FLASH_CHIP_SIZE "Taille flash"
#define D_FREE_PROGRAM_SPACE "Espace programme libre"

#define D_UPGRADE_BY_WEBSERVER "Mise à jour par serveur web"
#define D_OTA_URL "URL OTA"
#define D_START_UPGRADE "Lancer la mise à jour"
#define D_UPGRADE_BY_FILE_UPLOAD "Mise à jour par téléchargement fichier"
#define D_UPLOAD_FACTORY "Lancement de la partition Safeboot"
#define D_UPLOAD_STARTED "Téléchargement lancé"
#define D_UPGRADE_STARTED "Mise à jour lancée"
#define D_UPLOAD_DONE "Téléchargement terminé"
#define D_UPLOAD_TRANSFER "Transfert ZigBee"
#define D_TRANSFER_STARTED "Transfert lancé"
#define D_UPLOAD_ERR_1 "Aucun fichier sélectionné"
#define D_UPLOAD_ERR_2 "Espace insuffisant"
#define D_UPLOAD_ERR_3 "Signature de fichier invalide"
#define D_UPLOAD_ERR_4 "La taille du programme à flasher est plus grande que la taille réelle de la mémoire flash"
#define D_UPLOAD_ERR_5 "Erreur de comparaison du buffer de téléchargement"
#define D_UPLOAD_ERR_6 "Téléchargement échoué. Activer WebLog 3"
#define D_UPLOAD_ERR_7 "Téléchargement annulé"
#define D_UPLOAD_ERR_8 "Fichier invalide"
#define D_UPLOAD_ERR_9 "Fichier trop grand"
#define D_UPLOAD_ERR_10 "Erreur d'initialisation du chip RF"
#define D_UPLOAD_ERR_11 "Erreur d'effacement du chip RF"
#define D_UPLOAD_ERR_12 "Erreur d'accès en écriture au chip RF"
#define D_UPLOAD_ERR_13 "Erreur de décodage du firmware RF"
#define D_UPLOAD_ERR_14 "Incompatible"
#define D_UPLOAD_ERROR_CODE "Code d'erreur téléchargement"

#define D_ENTER_COMMAND "Saisir une commande"
#define D_ENABLE_WEBLOG_FOR_RESPONSE "Activer WebLog 2 si une réponse est attendue"
#define D_NEED_USER_AND_PASSWORD "Nécessite utilisateur=<username>&mot-de-passe=<password>"

// xdrv_01_mqtt.ino
#define D_FINGERPRINT "Vérification d'empreinte TLS ..."
#define D_TLS_CONNECT_FAILED_TO "Échec de connexion TLS à"
#define D_RETRY_IN "Nouvelle tentative dans"
#define D_VERIFIED "Vérifié par empreinte "
#define D_INSECURE "Connexion non sécurisée car empreinte non vérifiée"
#define D_CONNECT_FAILED_TO "Échec de connexion à"

// xplg_wemohue.ino
#define D_MULTICAST_DISABLED "Multicast désactivé"
#define D_MULTICAST_REJOINED "Multicast (re)joint"
#define D_MULTICAST_JOIN_FAILED "Multicast échec abonnement"
#define D_FAILED_TO_SEND_RESPONSE "Échec d'envoi réponse"

#define D_WEMO "WeMo"
#define D_WEMO_BASIC_EVENT "WeMo basic event"
#define D_WEMO_EVENT_SERVICE "WeMo event service"
#define D_WEMO_META_SERVICE "WeMo meta service"
#define D_WEMO_SETUP "Réglage WeMo"
#define D_RESPONSE_SENT "Réponse envoyée"

#define D_HUE "Hue"
#define D_HUE_BRIDGE_SETUP "Réglage Hue"
#define D_HUE_API_NOT_IMPLEMENTED "API Hue non implémentée"
#define D_HUE_API "Hue API"
#define D_HUE_POST_ARGS "Hue POST args"
#define D_3_RESPONSE_PACKETS_SENT "3 paquets de réponse envoyés"

// xdrv_07_domoticz.ino
#define D_DOMOTICZ_PARAMETERS "Paramètres Domoticz"
#define D_DOMOTICZ_IDX "Idx"
#define D_DOMOTICZ_KEY_IDX "Key idx"
#define D_DOMOTICZ_SWITCH_IDX "Switch idx"
#define D_DOMOTICZ_SENSOR_IDX "Sensor idx"
  #define D_DOMOTICZ_TEMP "Temp"
  #define D_DOMOTICZ_TEMP_HUM "Temp,Hum"
  #define D_DOMOTICZ_TEMP_HUM_BARO "Temp,Hum,Baro"
  #define D_DOMOTICZ_POWER_ENERGY "Puissance,Énergie"
  #define D_DOMOTICZ_ILLUMINANCE "Éclairement"
  #define D_DOMOTICZ_COUNT "Compteur/PM1"
  #define D_DOMOTICZ_VOLTAGE "Tension/PM2,5"
  #define D_DOMOTICZ_CURRENT "Courant/PM10"
  #define D_DOMOTICZ_AIRQUALITY "Qualité de l'air"
  #define D_DOMOTICZ_P1_SMART_METER "P1SmartMeter"
#define D_DOMOTICZ_UPDATE_TIMER "Durée de rafraichissement"

// xdrv_09_timers.ino
#define D_CONFIGURE_TIMER "Configuration des Timers"
#define D_TIMER_PARAMETERS "Paramètres Timer"
#define D_TIMER_ENABLE "Activer les Timers"
#define D_TIMER_ARM "Armer"
#define D_TIMER_TIME "Temps"
#define D_TIMER_DAYS "Jours"
#define D_TIMER_REPEAT "Répéter"
#define D_TIMER_OUTPUT "Sortie"
#define D_TIMER_ACTION "Action"

// xdrv_10_knx.ino
#define D_CONFIGURE_KNX "Configuration de KNX"
#define D_KNX_PARAMETERS "Paramètres KNX"
#define D_KNX_GENERAL_CONFIG "Général"
#define D_KNX_PHYSICAL_ADDRESS "Adresse individuelle"				// Backbone/Line/Device -> Zone/Ligne/Participant
#define D_KNX_PHYSICAL_ADDRESS_NOTE "(Doit être unique sur le réseau KNX)"
#define D_KNX_ENABLE "Activer KNX"
#define D_KNX_GROUP_ADDRESS_TO_WRITE "Données à envoyer aux Adresses de Groupe"
#define D_ADD "Ajoûter"
#define D_DELETE "Supprimer"
#define D_REPLY "Répondre"
#define D_KNX_GROUP_ADDRESS_TO_READ "Données à recevoir des Adresses de Groupe"
#define D_RECEIVED_FROM "Reçu de"
#define D_KNX_COMMAND_WRITE "Écrire"
#define D_KNX_COMMAND_READ "Lire"
#define D_KNX_COMMAND_OTHER "Autre"
#define D_SENT_TO "envoyé à"
#define D_KNX_WARNING "L'Adresse de Groupe (0/0/0) est réservée et ne peut être utilisée."
#define D_KNX_ENHANCEMENT "Amélioration de la communication"
#define D_KNX_TX_SLOT "KNX TX"
#define D_KNX_RX_SLOT "KNX RX"
#define D_KNX_TX_SCENE "KNX Scène TX"
#define D_KNX_RX_SCENE "KNX Scène RX"

// xdrv_23_zigbee
#define D_ZIGBEE_PERMITJOIN_ACTIVE "Modules autorisés à s'associer"
#define D_ZIGBEE_MAPPING_TITLE "Cartographie Tasmota ZigBee"
#define D_ZIGBEE_NOT_STARTED "ZigBee non démarré"
#define D_ZIGBEE_MAPPING_IN_PROGRESS_SEC "Cartographie en cours (%d s. restant)"
#define D_ZIGBEE_MAPPING_NOT_PRESENT "Pas de cartographie"
#define D_ZIGBEE_MAP_REFRESH "MàJ de la carto ZigBee"
#define D_ZIGBEE_MAP   "Carto ZigBee"
#define D_ZIGBEE_PERMITJOIN "Association ZigBee autorisée"
#define D_ZIGBEE_GENERATE_KEY "création d'une clé réseau ZigBee aléatoire"
#define D_ZIGBEE_UNKNOWN_DEVICE "Module inconnu"
#define D_ZIGBEE_UNKNOWN_ATTRIBUTE "Attribut inconnu"
#define D_ZIGBEE_UNKNOWN_ENDPOINT "Endpoint inconnu"
#define D_ZIGBEE_INVALID_PARAM "Paramètre invalide"
#define D_ZIGBEE_MISSING_PARAM "Paramètres manquants"
#define D_ZIGBEE_UNKNWON_ATTRIBUTE "Nom d'attribut inconnu (ignoré): %s"
#define D_ZIGBEE_TOO_MANY_CLUSTERS "Pas plus d'un Id de Cluster par commande"
#define D_ZIGBEE_CONFLICTING_ENDPOINTS "Conflit de Endpoint destinataires"
#define D_ZIGBEE_WRONG_DELIMITER "Mauvais délimiteur dans le contenu du message"
#define D_ZIGBEE_UNRECOGNIZED_COMMAND "Commande ZigBee inconnue: %s"
#define D_ZIGBEE_TOO_MANY_COMMANDS "Une seule commande autorisée (%d)"
#define D_ZIGBEE_NO_ATTRIBUTE "Liste sans attribut"
#define D_ZIGBEE_UNSUPPORTED_ATTRIBUTE_TYPE "Type d'attribut non supporté"
#define D_ZIGBEE_JSON_REQUIRED "Objets JSON requis par la config"
#define D_ZIGBEE_RESET_1_OR_2 "Réinitialiser 1 ou 2"
#define D_ZIGBEE_EEPROM_FOUND_AT_ADDRESS "EEPROM ZBBridge trouvée à l'adresse"
#define D_ZIGBEE_RANDOMIZING_ZBCONFIG "Randomisation des paramètres ZigBee, veuillez vérifier avec 'ZbConfig'"

// xdrv_89_dali.ino
#define D_SENSOR_DALI_RX                  "Dali RX"
#define D_SENSOR_DALI_TX                  "Dali TX"
#define D_CONFIGURE_DALI                  "Config DALI"

// xsns_03_energy.ino
#define D_ENERGY_TODAY "Énergie aujourd'hui"
#define D_ENERGY_YESTERDAY "Énergie hier"
#define D_ENERGY_TOTAL "Énergie totale"

// xdrv_27_shutter.ino
#define D_OPEN "Ouvert"
#define D_CLOSE "Fermé"
#define D_DOMOTICZ_SHUTTER "Volet"

// xdrv_28_pcf8574.ino
#define D_CONFIGURE_PCF8574 "Configurer PCF8574"
#define D_PCF8574_PARAMETERS "Paramètres PCF8574"
#define D_INVERT_PORTS "Inverser les Ports"
#define D_DEVICE "Module"
#define D_DEVICE_INPUT "Entrée"
#define D_DEVICE_OUTPUT "Sortie"

// xsns_05_ds18b20.ino
#define D_SENSOR_BUSY "Capteur occupé"
#define D_SENSOR_CRC_ERROR "Erreur CRC capteur"
#define D_SENSORS_FOUND "Capteur trouvé"

// xsns_06_dht.ino
#define D_TIMEOUT_WAITING_FOR "Expiration du délai"
#define D_START_SIGNAL_LOW "Signal Start bas"
#define D_START_SIGNAL_HIGH "Signal Start haut"
#define D_PULSE "Impulsion"
#define D_CHECKSUM_FAILURE "Erreur checksum"

// xsns_07_sht1x.ino
#define D_SENSOR_DID_NOT_ACK_COMMAND "Le capteur n'a pas acquitté la commande"
#define D_SHT1X_FOUND "SHT1X trouvé"

// xsns_18_pms5003.ino
#define D_STANDARD_CONCENTRATION "CF-1 PM"     // Standard Particle CF-1 Particle Matter
#define D_ENVIRONMENTAL_CONCENTRATION "PM"     // Environmetal Particle Matter
#define D_PARTICALS_BEYOND "Particules"

// xsns_27_apds9960.ino
#define D_GESTURE "Geste"
#define D_COLOR_RED "Rouge"
#define D_COLOR_GREEN "Vert"
#define D_COLOR_BLUE "Bleu"
#define D_CCT "CCT"
#define D_PROXIMITY "Proximité"

// xsns_32_mpu6050.ino
#define D_AX_AXIS "Accél. Axe-X"
#define D_AY_AXIS "Accél. Axe-Y"
#define D_AZ_AXIS "Accél. Axe-Z"
#define D_GX_AXIS "Gyro Axe-X"
#define D_GY_AXIS "Gyro Axe-Y"
#define D_GZ_AXIS "Gyro Axe-Z"

// xsns_33_QMC5883L.ino
#define D_MX "Induction X-Axis"
#define D_MY "Induction Y-Axis"
#define D_MZ "Induction Z-Axis"
#define D_MAGNETICFLD "Magnetic Induction"

// xsns_34_hx711.ino
#define D_HX_CAL_REMOVE "Retirer la charge"
#define D_HX_CAL_REFERENCE "Charger l'étalon de poids"
#define D_HX_CAL_DONE "Étalonné"
#define D_HX_CAL_FAIL "L'étalonnage a échoué"
#define D_RESET_HX711 "Réinitialiser le capteur"
#define D_CONFIGURE_HX711 "Configurer le capteur"
#define D_HX711_PARAMETERS "Paramètres capteur"
#define D_ITEM_WEIGHT "Poids de l'objet"
#define D_REFERENCE_WEIGHT "Poids de référence"
#define D_CALIBRATE "Étalonner"
#define D_CALIBRATION "Étalonnage"

// xsns_35_TX20.ino
#define D_TX20_WIND_DIRECTION "Direction du vent"
#define D_TX20_WIND_SPEED "Vitesse du vent"
#define D_TX20_WIND_SPEED_MIN "Vitesse Min"
#define D_TX20_WIND_SPEED_MAX "Vitesse Max"
#define D_TX20_NORTH "N"
#define D_TX20_EAST "E"
#define D_TX20_SOUTH "S"
#define D_TX20_WEST "O"

// xsns_53_sml.ino
#define D_TPWRIN "Energie totale Entrée"
#define D_TPWRIN0 "Energie totale Entrée T0"
#define D_TPWRIN1 "Energie totale Entrée T1"
#define D_TPWROUT "Energie totale Sortie"
#define D_TPWRCURR "Puissance active E/S"
#define D_TPWRCURR1 "Puissance active Ent Ph1"
#define D_TPWRCURR2 "Puissance active Ent Ph2"
#define D_TPWRCURR3 "Puissance active Ent Ph3"
#define D_Strom_L1 "Courant Ph1"
#define D_Strom_L2 "Courant Ph2"
#define D_Strom_L3 "Courant Ph3"
#define D_Spannung_L1 "Tension Ph1"
#define D_Spannung_L2 "Tension Ph2"
#define D_Spannung_L3 "Tension Ph3"
#define D_METERNR "Numéro compteur"
#define D_METERSID "Service ID"
#define D_GasIN "Compteur Gaz"
#define D_H2oIN "Compteur Eau"
#define D_StL1L2L3 "Courant Ph1+Ph2+Ph3"
#define D_SpL1L2L3 "Tension (Ph1+Ph2+Ph3)/3"

// xsns_88_tfminiplus.ino
#define D_SIGNALSTRENGTH "Signal Strength"
#define D_CHIPTEMPERATURE "Chip Temperature"

// tasmota_template.h - keep them as short as possible to be able to fit them in GUI drop down box
#define D_SENSOR_NONE          "Aucun"
#define D_SENSOR_USER          "Utilisateur"
#define D_SENSOR_OPTION        "Option"
#define D_SENSOR_DHT11         "DHT11"
#define D_SENSOR_AM2301        "AM2301"
#define D_SENSOR_SI7021        "SI7021"
#define D_SENSOR_MS01          "MS01"
#define D_SENSOR_DS18X20       "DS18x20"
#define D_SENSOR_I2C_SCL       "I2C SCl"
#define D_SENSOR_I2C_SDA       "I2C SDa"
#define D_SENSOR_I2S_MCLK      "I2S MClk"
#define D_SENSOR_I2S_BCLK      "I2S BClk"
#define D_SENSOR_I2S_WS_IN     "I2S BClk In"
#define D_SENSOR_I2S_WS        "I2S WS"
#define D_SENSOR_I2S_BCLK_IN   "I2S WS In"
#define D_SENSOR_I2S_DIN       "I2S DIn"
#define D_SENSOR_I2S_DOUT      "I2S DOut"
#define D_SENSOR_I2S_DAC       "I2S DAC"
#define D_SENSOR_HDMI_CEC      "HDMI CEC"
#define D_SENSOR_WS2812        "WS2812"
#define D_SENSOR_DFR562        "MP3 Player"
#define D_SENSOR_DFR562_BUSY   "MP3 Busy"
#define D_SENSOR_IRSEND        "IR Tx"
#define D_SENSOR_SWITCH        "Inter"      // Suffix "1"
#define D_SENSOR_BUTTON        "Bouton"     // Suffix "1"
#define D_SENSOR_RELAY         "Relais"     // Suffix "1i"
#define D_SENSOR_LED           "LED"        // Suffix "1i"
#define D_SENSOR_LED_LINK      "LedLink"    // Suffix "i"
#define D_SENSOR_PWM           "PWM"        // Suffix "1"
#define D_SENSOR_COUNTER       "Compteur"   // Suffix "1"
#define D_SENSOR_INTERRUPT     "Interrupt"
#define D_SENSOR_INPUT         "Input"
#define D_SENSOR_IRRECV        "IR Rx"
#define D_SENSOR_MHZ_RX        "MHZ Rx"
#define D_SENSOR_MHZ_TX        "MHZ Tx"
#define D_SENSOR_HC8_RX        "HC8 Rx"
#define D_SENSOR_PZEM004_RX    "PZEM004 Rx"
#define D_SENSOR_PZEM016_RX    "PZEM016 Rx"
#define D_SENSOR_PZEM017_RX    "PZEM017 Rx"
#define D_SENSOR_PZEM0XX_TX    "PZEM0XX Tx"
#define D_SENSOR_SAIR_RX       "SAir Rx"
#define D_SENSOR_SAIR_TX       "SAir Tx"
#define D_SENSOR_SPI_CS        "SPI CS"
#define D_SENSOR_SPI_DC        "SPI DC"
#define D_SENSOR_SPI_MISO      "SPI MISO"
#define D_SENSOR_SPI_MOSI      "SPI MOSI"
#define D_SENSOR_SPI_CLK       "SPI Clk"
#define D_SENSOR_SDIO_CMD      "SDIO Cmd"
#define D_SENSOR_SDIO_CLK      "SDIO Clk"
#define D_SENSOR_SDIO_D0       "SDIO D0"
#define D_SENSOR_SDIO_D1       "SDIO D1"
#define D_SENSOR_SDIO_D2       "SDIO D2"
#define D_SENSOR_SDIO_D3       "SDIO D3"
#define D_SENSOR_BACKLIGHT     "RétroÉcl"
#define D_SENSOR_PMS5003_TX    "PMS5003 Tx"
#define D_SENSOR_PMS5003_RX    "PMS5003 Rx"
#define D_SENSOR_SDS0X1_RX     "SDS0X1 Rx"
#define D_SENSOR_SDS0X1_TX     "SDS0X1 Tx"
#define D_SENSOR_HPMA_RX       "HPMA Rx"
#define D_SENSOR_HPMA_TX       "HPMA Tx"
#define D_SENSOR_SBR_RX        "SerBr Rx"
#define D_SENSOR_SBR_TX        "SerBr Tx"
#define D_SENSOR_MBR_TX        "ModBr Tx"
#define D_SENSOR_MBR_RX        "ModBr Rx"
#define D_SENSOR_MBR_TX_ENA    "ModBr Tx Ena"
#define D_SENSOR_SR04_TRIG     "SR04 Tri/Tx"
#define D_SENSOR_SR04_ECHO     "SR04 Ech/Rx"
#define D_SENSOR_NRG_MBS_TX    "NrgMbus Tx"
#define D_SENSOR_NRG_MBS_RX    "NrgMbus Rx"
#define D_SENSOR_NRG_MBS_TX_ENA "NrgMbus Tx Ena"
#define D_SENSOR_SDM72_TX      "SDM72 Tx"
#define D_SENSOR_SDM72_RX      "SDM72 Rx"
#define D_SENSOR_SDM120_TX     "SDMx20 Tx"
#define D_SENSOR_SDM120_RX     "SDMx20 Rx"
#define D_SENSOR_SDM230_TX     "SDM230 Tx"
#define D_SENSOR_SDM230_RX     "SDM230 Rx"
#define D_SENSOR_SDM630_TX     "SDM630 Tx"
#define D_SENSOR_SDM630_RX     "SDM630 Rx"
#define D_SENSOR_WE517_TX      "WE517 Tx"
#define D_SENSOR_WE517_RX      "WE517 Rx"
#define D_SENSOR_LD2410_TX     "LD2410 Tx"
#define D_SENSOR_LD2410_RX     "LD2410 Rx"
#define D_GPIO_TM1621_CS       "TM1621 CS"
#define D_GPIO_TM1621_WR       "TM1621 Wr"
#define D_GPIO_TM1621_RD       "TM1621 Rd"
#define D_GPIO_TM1621_DAT      "TM1621 Dat"
#define D_SENSOR_TM1637_CLK    "TM1637 Clk"
#define D_SENSOR_TM1637_DIO    "TM1637 DIO"
#define D_SENSOR_TM1638_CLK    "TM1638 Clk"
#define D_SENSOR_TM1638_DIO    "TM1638 DIO"
#define D_SENSOR_TM1638_STB    "TM1638 Stb"
#define D_SENSOR_MAX7219_DIN   "MAX7219 Din"
#define D_SENSOR_MAX7219_CS    "MAX7219 CS"
#define D_SENSOR_MAX7219_CLK   "MAX7219 Clk"
#define D_SENSOR_HX711_SCK     "HX711 SCk"
#define D_SENSOR_HX711_DAT     "HX711 Dat"
#define D_SENSOR_FTC532        "FTC532"
#define D_SENSOR_BS814_CLK     "BS814 Clk"
#define D_SENSOR_BS814_DAT     "BS814 Dat"
#define D_SENSOR_TX2X_TX       "TX2x"
#define D_SENSOR_RFSEND        "RF Tx"
#define D_SENSOR_RFRECV        "RF Rx"
#define D_SENSOR_TUYA_TX       "Tuya Tx"
#define D_SENSOR_TUYA_RX       "Tuya Rx"
#define D_SENSOR_MGC3130_XFER  "MGC3130 Xfr"
#define D_SENSOR_MGC3130_RESET "MGC3130 Rst"
#define D_SENSOR_SSPI_MISO     "SSPI MISO"
#define D_SENSOR_SSPI_MOSI     "SSPI MOSI"
#define D_SENSOR_SSPI_SCLK     "SSPI SClk"
#define D_SENSOR_SSPI_CS       "SSPI CS"
#define D_SENSOR_SSPI_DC       "SSPI DC"
#define D_SENSOR_RF_SENSOR     "Capteur RF"
#define D_SENSOR_AZ_RX         "AZ Rx"
#define D_SENSOR_AZ_TX         "AZ Tx"
#define D_SENSOR_MAX31855_CS   "MX31855 CS"
#define D_SENSOR_MAX31855_CLK  "MX31855 Clk"
#define D_SENSOR_MAX31855_DO   "MX31855 DO"
#define D_SENSOR_MAX31865_CS   "MX31865 CS"
#define D_SENSOR_NRG_SEL       "HLWBL Sel"  // Suffix "i"
#define D_SENSOR_NRG_CF1       "HLWBL CF1"
#define D_SENSOR_HLW_CF        "HLW8012 CF"
#define D_SENSOR_HJL_CF        "BL0937 CF"
#define D_SENSOR_MCP39F5_TX    "MCP39F5 Tx"
#define D_SENSOR_MCP39F5_RX    "MCP39F5 Rx"
#define D_SENSOR_MCP39F5_RST   "MCP39F5 Rst"
#define D_SENSOR_CSE7761_TX    "CSE7761 Tx"
#define D_SENSOR_CSE7761_RX    "CSE7761 Rx"
#define D_SENSOR_CSE7766_TX    "CSE7766 Tx"
#define D_SENSOR_CSE7766_RX    "CSE7766 Rx"
#define D_SENSOR_BL0939_RX     "BL0939 Rx"
#define D_SENSOR_BL0942_RX     "BL0942 Rx"
#define D_SENSOR_HM330X_SET    "HM330X Set"
#define D_SENSOR_PN532_TX      "PN532 Tx"
#define D_SENSOR_PN532_RX      "PN532 Rx"
#define D_SENSOR_SM16716_CLK   "SM16716 Clk"
#define D_SENSOR_SM16716_DAT   "SM16716 Dat"
#define D_SENSOR_SM16716_POWER "SM16716 Pwr"
#define D_SENSOR_P9813_CLK     "P9813 Clk"
#define D_SENSOR_P9813_DAT     "P9813 Dat"
#define D_SENSOR_MY92X1_DI     "MY92x1 DI"
#define D_SENSOR_MY92X1_DCKI   "MY92x1 DCkI"
#define D_SENSOR_ARIRFRCV      "ALux IrRcv"
#define D_SENSOR_ARIRFSEL      "ALux IrSel"
#define D_SENSOR_TXD           "Série Tx"
#define D_SENSOR_RXD           "Série Rx"
#define D_SENSOR_ROTARY        "Rotary"     // Suffix "1A"
#define D_SENSOR_HRE_CLOCK     "HRE Clk"
#define D_SENSOR_HRE_DATA      "HRE Dat"
#define D_SENSOR_ADE7880_IRQ   "ADE7880 IRq"
#define D_SENSOR_ADE7953_IRQ   "ADE7953 IRq"
#define D_SENSOR_ADE7953_RST   "ADE7953 Rst"
#define D_SENSOR_ADE7953_CS    "ADE7953 CS"
#define D_SENSOR_BUZZER        "Buzzer"
#define D_SENSOR_DISP_RESET    "Display Rst"
#define D_SENSOR_ZIGBEE_TXD    "ZigBee Tx"
#define D_SENSOR_ZIGBEE_RXD    "ZigBee Rx"
#define D_SENSOR_ZIGBEE_RST    "ZigBee Rst"
#define D_SENSOR_SOLAXX1_TX    "SolaxX1 Tx"
#define D_SENSOR_SOLAXX1_RX    "SolaxX1 Rx"
#define D_SENSOR_SOLAXX1_RTS   "SolaxX1 RTS"
#define D_SENSOR_IBEACON_TX    "iBeacon Tx"
#define D_SENSOR_IBEACON_RX    "iBeacon Rx"
#define D_SENSOR_RDM6300_RX    "RDM6300 Rx"
#define D_SENSOR_CC1101_CS     "CC1101 CS"
#define D_SENSOR_A4988_DIR     "A4988 Dir"
#define D_SENSOR_A4988_STP     "A4988 Stp"
#define D_SENSOR_A4988_ENA     "A4988 Ena"
#define D_SENSOR_A4988_MS1     "A4988 MS1"
#define D_SENSOR_OUTPUT_HI     "Sortie Hi"
#define D_SENSOR_OUTPUT_LO     "Sortie Lo"
#define D_SENSOR_AS608_TX      "AS608 Tx"
#define D_SENSOR_AS608_RX      "AS608 Rx"
#define D_SENSOR_GM861_TX      "GM861 Tx"
#define D_SENSOR_GM861_RX      "GM861 Rx"
#define D_SENSOR_DDS2382_TX    "DDS238-2 Tx"
#define D_SENSOR_DDS2382_RX    "DDS238-2 Rx"
#define D_SENSOR_DDSU666_TX    "DDSU666 Tx"
#define D_SENSOR_DDSU666_RX    "DDSU666 Rx"
#define D_SENSOR_SM2135_CLK    "SM2135 Clk"
#define D_SENSOR_SM2135_DAT    "SM2135 Dat"
#define D_SENSOR_SM2335_CLK    "SM2335 Clk"
#define D_SENSOR_SM2335_DAT    "SM2335 Dat"
#define D_SENSOR_BP1658CJ_CLK  "BP1658CJ Clk"
#define D_SENSOR_BP1658CJ_DAT  "BP1658CJ Dat"
#define D_SENSOR_BP5758D_CLK   "BP5758D Clk"
#define D_SENSOR_BP5758D_DAT   "BP5758D Dat"
#define D_SENSOR_DEEPSLEEP     "Hibernation"
#define D_SENSOR_EXS_ENABLE    "EXS Enable"
#define D_SENSOR_CLIENT_TX     "Esclave Tx"
#define D_SENSOR_CLIENT_RX     "Esclave Rx"
#define D_SENSOR_CLIENT_RESET  "Esclave RST"
#define D_SENSOR_GPS_RX        "GPS Rx"
#define D_SENSOR_GPS_TX        "GPS Tx"
#define D_SENSOR_HM10_RX       "HM10 Rx"
#define D_SENSOR_HM10_TX       "HM10 Tx"
#define D_SENSOR_LE01MR_RX     "LE-01MR Rx"
#define D_SENSOR_LE01MR_TX     "LE-01MR Tx"
#define D_SENSOR_BL0940_RX     "BL0940 Rx"
#define D_SENSOR_CC1101_GDO0   "CC1101 GDO0"
#define D_SENSOR_CC1101_GDO2   "CC1101 GDO2"
#define D_SENSOR_HRXL_RX       "HRXL Rx"
#define D_SENSOR_DYP_RX        "DYP Rx"
#define D_SENSOR_ELECTRIQ_MOODL "MOODL Tx"
#define D_SENSOR_AS3935        "AS3935"
#define D_SENSOR_WINDMETER_SPEED "Anémomètre"
#define D_SENSOR_TELEINFO_RX   "TInfo Rx"
#define D_SENSOR_TELEINFO_ENABLE "TInfo En"
#define D_SENSOR_LMT01_PULSE   "LMT01 Impulsion"
#define D_SENSOR_ADC_INPUT     "ADC Entrée"
#define D_SENSOR_ADC_TEMP      "ADC Temp."
#define D_SENSOR_ADC_LIGHT     "ADC Lumière"
#define D_SENSOR_ADC_BUTTON    "ADC Bouton"
#define D_SENSOR_ADC_RANGE     "ADC Distance"
#define D_SENSOR_ADC_CT_POWER  "ADC CT Power"
#define D_SENSOR_ADC_JOYSTICK  "ADC Manette"
#define D_SENSOR_ADC_PH        "ADC pH"
#define D_SENSOR_ADC_MQ        "ADC MQ"
#define D_GPIO_WEBCAM_PWDN     "CAM_PwDn"
#define D_GPIO_WEBCAM_RESET    "CAM_Reset"
#define D_GPIO_WEBCAM_XCLK     "CAM_XClk"
#define D_GPIO_WEBCAM_SIOD     "CAM_SIOD"
#define D_GPIO_WEBCAM_SIOC     "CAM_SIOC"
#define D_GPIO_WEBCAM_DATA     "CAM_Data"
#define D_GPIO_WEBCAM_VSYNC    "CAM_VSync"
#define D_GPIO_WEBCAM_HREF     "CAM_HRef"
#define D_GPIO_WEBCAM_PCLK     "CAM_PClk"
#define D_GPIO_WEBCAM_PSCLK    "CAM_PSClk"
#define D_GPIO_WEBCAM_HSD      "CAM_HSD"
#define D_GPIO_WEBCAM_PSRCS    "CAM_PSRCS"
#define D_SENSOR_ETH_PHY_POWER "ETH Pwr"
#define D_SENSOR_ETH_PHY_MDC   "ETH MDC"
#define D_SENSOR_ETH_PHY_MDIO  "ETH MDIO"
#define D_SENSOR_TCP_TXD       "TCP Tx"
#define D_SENSOR_TCP_RXD       "TCP Rx"
#define D_SENSOR_IEM3000_TX    "iEM3000 Tx"
#define D_SENSOR_IEM3000_RX    "iEM3000 Rx"
#define D_SENSOR_MIEL_HVAC_TX  "MiEl HVAC Tx"
#define D_SENSOR_MIEL_HVAC_RX  "MiEl HVAC Rx"
#define D_SENSOR_PROJECTOR_CTRL_TX  "DLP Tx"
#define D_SENSOR_PROJECTOR_CTRL_RX  "DLP Rx"
#define D_SENSOR_SHELLY_DIMMER_BOOT0 "SHD Boot0"
#define D_SENSOR_SHELLY_DIMMER_RST_INV "SHD Rst"
#define D_SENSOR_RC522_RST     "RC522 Rst"
#define D_SENSOR_RC522_CS      "RC522 CS"
#define D_SENSOR_NRF24_CS      "NRF24 CS"
#define D_SENSOR_NRF24_DC      "NRF24 DC"
#define D_SENSOR_ILI9341_CS    "ILI9341 CS"
#define D_SENSOR_ILI9341_DC    "ILI9341 DC"
#define D_SENSOR_ILI9488_CS    "ILI9488 CS"
#define D_SENSOR_EPAPER29_CS   "EPaper29 CS"
#define D_SENSOR_XPT2046_CS    "XPT2046 CS"
#define D_SENSOR_EPAPER42_CS   "EPaper42 CS"
#define D_SENSOR_SSD1351_CS    "SSD1351 CS"
#define D_SENSOR_SSD1351_DC    "SSD1351 DC"
#define D_SENSOR_RA8876_CS     "RA8876 CS"
#define D_SENSOR_ST7789_CS     "ST7789 CS"
#define D_SENSOR_ST7789_DC     "ST7789 DC"
#define D_SENSOR_SSD1331_CS    "SSD1331 CS"
#define D_SENSOR_SSD1331_DC    "SSD1331 DC"
#define D_SENSOR_SDCARD_CS     "CarteSD CS"
#define D_SENSOR_WIEGAND_D0    "Wiegand D0"
#define D_SENSOR_WIEGAND_D1    "Wiegand D1"
#define D_SENSOR_NEOPOOL_TX    "NeoPool Tx"
#define D_SENSOR_NEOPOOL_RX    "NeoPool Rx"
#define D_SENSOR_VL53LXX_XSHUT "VL53LXX XShut"
#define D_SENSOR_TFMINIPLUS_TX  "TFmini+ Tx"
#define D_SENSOR_TFMINIPLUS_RX  "TFmini+ Rx"
#define D_SENSOR_ZEROCROSS     "ZC Pulse"
#define D_SENSOR_HALLEFFECT    "Effet Hall"
#define D_SENSOR_EPD_DATA      "EPD Data"
#define D_SENSOR_PCF8574_INT   "PCF8574 Int"
#define D_SENSOR_MCP23XXX_INT  "MCP23xxx Int"
#define D_SENSOR_MCP23SXX_CS   "MCP23Sxx CS"
#define D_SENSOR_MCP2515_CS    "MCP2515 CS"
#define D_SENSOR_HRG15_RX      "HRG15 Rx"
#define D_SENSOR_HRG15_TX      "HRG15 Tx"
#define D_SENSOR_VINDRIKTNING_RX "Vindriktning"
#define D_SENSOR_BL6523_TX     "BL6523 Tx"
#define D_SENSOR_BL6523_RX     "BL6523 Rx"
#define D_SENSOR_HEARTBEAT     "Heartbeat"
#define D_SENSOR_RESET         "Reset"
#define D_GPIO_SHIFT595_SRCLK  "74x595 SRClk"
#define D_GPIO_SHIFT595_RCLK   "74x595 RClk"
#define D_GPIO_SHIFT595_OE     "74x595 OE"
#define D_GPIO_SHIFT595_SER    "74x595 Ser"
#define D_GPIO_DINGTIAN_CLK    "Dingtian Clk"
#define D_GPIO_DINGTIAN_SDI    "Dingtian SDI"
#define D_GPIO_DINGTIAN_Q7     "Dingtian Q7"
#define D_GPIO_DINGTIAN_PL     "Dingtian PL"
#define D_GPIO_DINGTIAN_OE     "Dingtian OE"
#define D_GPIO_DINGTIAN_RCK    "Dingtian RCk"
#define D_SENSOR_CM11_TX       "CM110x Tx"
#define D_SENSOR_CM11_RX       "CM110x Rx"
#define D_SENSOR_FLOWRATEMETER "Débit"
#define D_SENSOR_ME007_TRIG    "ME007 Tri"
#define D_SENSOR_ME007_RX      "ME007 Rx"
#define D_SENSOR_TUYAMCUBR_TX  "TuyaMCUBr Tx"
#define D_SENSOR_TUYAMCUBR_RX  "TuyaMCUBr Rx"
#define D_SENSOR_BIOPDU_PZEM0XX_TX "BioPDU PZEM0XX Tx"
#define D_SENSOR_BIOPDU_PZEM016_RX "BioPDU PZEM016 Rx"
#define D_SENSOR_BIOPDU_BIT    "BioPDU Bit"
#define D_SENSOR_LOX_O2_RX     "LoxO2 RX"


// Units
#define D_UNIT_AMPERE "A"
#define D_UNIT_CELSIUS "C"
#define D_UNIT_CENTIMETER "cm"
#define D_UNIT_DEGREE "°"
#define D_UNIT_FAHRENHEIT "F"
#define D_UNIT_FPS "FPS"
#define D_UNIT_HERTZ "Hz"
#define D_UNIT_HOUR "h"
#define D_UNIT_GALLONS "gal"
#define D_UNIT_GALLONS_PER_MIN "gal/min"
#define D_UNIT_GRAM_PER_CUBIC_METER "g/m³"
#define D_UNIT_INCH_MERCURY "inHg"
#define D_UNIT_INCREMENTS "inc"
#define D_UNIT_KELVIN "K"
#define D_UNIT_KILOMETER "km"
#define D_UNIT_KILOGRAM "kg"
#define D_UNIT_KILOMETER_PER_HOUR "km/h"
#define D_UNIT_KILOOHM "kΩ"
#define D_UNIT_KILOWATTHOUR "kWh"
#define D_UNIT_LITERS "L"
#define D_UNIT_LITERS_PER_MIN "L/min"
#define D_UNIT_LUX "lx"
#define D_UNIT_MICROGRAM_PER_CUBIC_METER "µg/m³"
#define D_UNIT_MICROMETER "µm"
#define D_UNIT_MICROSECOND "µs"
#define D_UNIT_MICROSIEMENS_PER_CM "µS/cm"
#define D_UNIT_MICROTESLA "µT"
#define D_UNIT_MILLIAMPERE "mA"
#define D_UNIT_MILLILITERS "ml"
#define D_UNIT_MILLIMETER "mm"
#define D_UNIT_MILLIMETER_MERCURY "mmHg"
#define D_UNIT_MILLISECOND "ms"
#define D_UNIT_MILLIVOLT "mV"
#define D_UNIT_MINUTE "min"  // https://fr.wikipedia.org/wiki/Minute_(temps)#Symbole%20et%20d%C3%A9finition
#define D_UNIT_PARTS_PER_BILLION "ppb"
#define D_UNIT_PARTS_PER_DECILITER "ppd"
#define D_UNIT_PARTS_PER_MILLION "ppm"
#define D_UNIT_MILIGRAMS_PER_LITER "mg/L"
#define D_UNIT_PERCENT "%%"
#define D_UNIT_PRESSURE "hPa"
#define D_UNIT_SECOND "s"
#define D_UNIT_SECTORS "secteurs"
#define D_UNIT_VA "VA"
#define D_UNIT_VAR "VAr"
#define D_UNIT_VOLT "V"
#define D_UNIT_WATT "W"
#define D_UNIT_WATTHOUR "Wh"
#define D_UNIT_WATT_METER_QUADRAT "W/m²"
#define D_UNIT_LITER_PER_MINUTE "l/min"
#define D_UNIT_CUBICMETER_PER_HOUR "m³/h"
#define D_UNIT_CUBIC_METER "m³"

#define D_NEW_ADDRESS     "Positionner l'adresse à"
#define D_OUT_OF_RANGE    "Hors limites"
#define D_SENSOR_DETECTED "détecté"

//SDM220, SDM120, SDM72, LE01MR, SDM230
#define D_EXPORT_POWER    "Puissance fournie"
#define D_IMPORT_POWER 	  "Puissance consommée"
#define D_PHASE_ANGLE     "Angle de phase"
#define D_IMPORT_ACTIVE   "Énergie act conso"
#define D_EXPORT_ACTIVE   "Énergie act fournie"
#define D_IMPORT_REACTIVE "Énergie réa conso"
#define D_EXPORT_REACTIVE "Énergie réa fournie"
#define D_TOTAL_REACTIVE  "Énergie réa totale"
#define D_UNIT_KWARH      "kVArh"
#define D_UNIT_ANGLE      "°"
#define D_TOTAL_ACTIVE    "Total Active"
#define D_RESETTABLE_TOTAL_ACTIVE    "Total Active (RST)"

//SOLAXX1
#define D_PV1_VOLTAGE     "Tension PV1"
#define D_PV1_CURRENT     "Courant PV1"
#define D_PV1_POWER       "Puissance PV1"
#define D_PV2_VOLTAGE     "Tension PV2"
#define D_PV2_CURRENT     "Courant PV2"
#define D_PV2_POWER       "Puissance PV2"
#define D_SOLAR_POWER     "Puissance solaire"
#define D_INVERTER_POWER  "Puissance onduleur"
#define D_STATUS          "Statut"
#define D_SOLAX_MODE_0    "En attente"
#define D_SOLAX_MODE_1    "En test"
#define D_SOLAX_MODE_2    "En marche"
#define D_SOLAX_MODE_3    "Défault"
#define D_SOLAX_MODE_4    "Défaut permanent"
#define D_SOLAX_MODE_5    "MàJ logicielle"
#define D_SOLAX_MODE_6    "Autotest"
#define D_SOLAX_ERROR_0   "Aucun code d'erreur"
#define D_SOLAX_ERROR_1   "Défaut Perte de réseau"
#define D_SOLAX_ERROR_2   "Défaut Tension réseau"
#define D_SOLAX_ERROR_3   "Défaut Fréquence réseau"
#define D_SOLAX_ERROR_4   "Défaut Tension PV"
#define D_SOLAX_ERROR_5   "Défaut Isolation"
#define D_SOLAX_ERROR_6   "Défaut Surchauffe"
#define D_SOLAX_ERROR_7   "Défaut Ventilateur"
#define D_SOLAX_ERROR_8   "Défaut Autre équipement"

//xdrv_10_scripter.ino
#define D_CONFIGURE_SCRIPT     "Éditer le script"
#define D_SCRIPT               "édition du script"
#define D_SDCARD_UPLOAD        "Envoi du fichier"
#define D_UFSDIR               "Dossier UFS"
#define D_UPL_DONE             "Terminé"
#define D_SCRIPT_CHARS_LEFT    "car. restant"
#define D_SCRIPT_CHARS_NO_MORE "plus de car."
#define D_SCRIPT_DOWNLOAD      "Download"
#define D_SCRIPT_ENABLE        "script actif"
#define D_SCRIPT_UPLOAD        "Envoi"
#define D_SCRIPT_UPLOAD_FILES  "Envoi de fichiers"

//xdrv_50_filesystem.ino
#define D_MANAGE_FILE_SYSTEM   "Gestion du Système de Fichier"
#define D_FS_SIZE              "Taille"
#define D_FS_FREE              "Libre"
#define D_NEW_FILE             "nouveau-fichier.txt"
#define D_CREATE_NEW_FILE      "Créer and modifier un nouveau fichier"
#define D_EDIT_FILE            "Modification de fichier"
#define D_CONFIRM_FILE_DEL     "Confirmez la suppression du fichier"
#define D_SHOW_HIDDEN_FILES    "Affichez les fichiers cachés"

//xsns_67_as3935.ino
#define D_AS3935_GAIN "gain:"
#define D_AS3935_ENERGY "energie:"
#define D_AS3935_DISTANCE "distance:"
#define D_AS3935_DISTURBER "interférence:"
#define D_AS3935_VRMS "µVrms:"
#define D_AS3935_APRX "approx.:"
#define D_AS3935_AWAY "de distance"
#define D_AS3935_LIGHT "éclair"
#define D_AS3935_OUT "éclair éloigné"
#define D_AS3935_NOT "distance indéterminée"
#define D_AS3935_ABOVE "éclair trop intense"
#define D_AS3935_NOISE "bruit détecté"
#define D_AS3935_DISTDET "interférence détectée"
#define D_AS3935_INTNOEV "Interruption sans évenement!"
#define D_AS3935_FLICKER "IRQ flicker!"
#define D_AS3935_POWEROFF "Power Off"
#define D_AS3935_NOMESS "en écoute..."
#define D_AS3935_ON "Actif"
#define D_AS3935_OFF "Inactif"
#define D_AS3935_INDOORS "Intérieur"
#define D_AS3935_OUTDOORS "Extérieur"
#define D_AS3935_CAL_FAIL "défaut de calibration"
#define D_AS3935_CAL_OK "calibration établie à :"

//xsns_68_opentherm.ino
#define D_SENSOR_BOILER_OT_RX   "OpenTherm RX"
#define D_SENSOR_BOILER_OT_TX   "OpenTherm TX"

// xnrg_15_teleinfo Denky (Teleinfo)
#define D_CONTRACT        "Type de contrat"
#define D_POWER_LOAD      "Charge actuelle"
#define D_CURRENT_TARIFF  "Tarif en cours"
#define D_TARIFF          "Tarif"
#define D_OVERLOAD        "ADPS"
#define D_MAX_POWER       "Puissance max"
#define D_MAX_CURRENT     "Courant max"

// xsns_79_as608.ino
#define D_FP_ENROLL_PLACEFINGER "Placer un doigt"
#define D_FP_ENROLL_REMOVEFINGER "Retirer le doigt"
#define D_FP_ENROLL_PLACESAMEFINGER "Replacer le même doigt"
#define D_FP_ENROLL_RETRY "Erreur, rééssayer"
#define D_FP_ENROLL_RESTART "Redémarrer"
#define D_FP_ENROLL_ERROR "Erreur"
#define D_FP_ENROLL_RESET "Réinitialiser"
#define D_FP_ENROLL_ACTIVE "Actif"
#define D_FP_ENROLL_INACTIVE "Inactif"
// Indexed by Adafruit_Fingerprint.h defines
#define D_FP_PACKETRECIEVEERR "Erreur de com." // 0x01 Error when receiving data package
#define D_FP_NOFINGER ""                       // 0x02 No finger on the sensor
#define D_FP_IMAGEFAIL "Erreur d'acquisition"  // 0x03 Failed to enroll the finger
#define D_FP_IMAGEMESS "Empreinte incohérante" // 0x06 Failed to generate character file due to overly disorderly fingerprint image
#define D_FP_FEATUREFAIL "Empreinte trop petite" // 0x07 Failed to generate character file due to the lack of character point or small fingerprint image
#define D_FP_NOMATCH "Le doigt ne correspond pas" // 0x08 Finger doesn't match
#define D_FP_NOTFOUND "Pas de doigt correspondant" // 0x09 Failed to find matching finger
#define D_FP_ENROLLMISMATCH "Échec de la comparaison" // 0x0A Failed to combine the character files
#define D_FP_BADLOCATION "Erreur d'indexation" // 0x0B Addressed PageID is beyond the finger library
#define D_FP_DBRANGEFAIL "Modèle invalide"     // 0x0C Error when reading template from library or invalid template
#define D_FP_UPLOADFEATUREFAIL "Erreur de transfert" // 0x0D Error when uploading template
#define D_FP_PACKETRESPONSEFAIL "Transfert interrompu" // 0x0E Module failed to receive the following data packages
#define D_FP_UPLOADFAIL "Transfert échoué"     // 0x0F Error when uploading image
#define D_FP_DELETEFAIL "Suppression échouée"  // 0x10 Failed to delete the template
#define D_FP_DBCLEARFAIL "Réinitialisation échouée" // 0x11 Failed to clear finger library
#define D_FP_PASSFAIL "Mot-de-passe erroné"    // 0x13 Find whether the fingerprint passed or failed
#define D_FP_INVALIDIMAGE "Image incorrecte"   // 0x15 Failed to generate image because of lack of valid primary image
#define D_FP_FLASHERR "Erreur d'écriture en Flash" // 0x18 Error when writing flash
#define D_FP_INVALIDREG "Nombre incorrect"     // 0x1A Invalid register number
#define D_FP_ADDRCODE "Code adresse"           // 0x20 Address code
#define D_FP_PASSVERIFY "Mot-de-passe vérifié" // 0x21 Verify the fingerprint passed
#define D_FP_UNKNOWNERROR "Erreur"             // Any other error

// xsns_96_flowratemeter.ino
#define D_FLOWRATEMETER_NAME "Débit"
#define D_FLOWRATEMETER_AMOUNT_TODAY "Quantité aujourd'hui"
#define D_FLOWRATEMETER_DURATION_TODAY "Durée aujourd'hui"

// xsns_83_neopool.ino
#define D_NEOPOOL_MACH_NONE               "NeoPool"           // Machine names
#define D_NEOPOOL_MACH_HIDROLIFE          "Hidrolife"
#define D_NEOPOOL_MACH_AQUASCENIC         "Aquascenic"
#define D_NEOPOOL_MACH_OXILIFE            "Oxilife"
#define D_NEOPOOL_MACH_BIONET             "Bionet"
#define D_NEOPOOL_MACH_HIDRONISER         "Hidroniser"
#define D_NEOPOOL_MACH_UVSCENIC           "UVScenic"
#define D_NEOPOOL_MACH_STATION            "Station"
#define D_NEOPOOL_MACH_BRILIX             "Brilix"
#define D_NEOPOOL_MACH_GENERIC            "Generic"
#define D_NEOPOOL_MACH_BAYROL             "Bayrol"
#define D_NEOPOOL_MACH_HAY                "Hay"
#define D_NEOPOOL_FILTRATION_MANUAL       "Manuel"            // Filtration modes
#define D_NEOPOOL_FILTRATION_AUTO         "Auto"
#define D_NEOPOOL_FILTRATION_HEATING      "Chauffage"
#define D_NEOPOOL_FILTRATION_SMART        "Malin"
#define D_NEOPOOL_FILTRATION_INTELLIGENT  "Intelligent"
#define D_NEOPOOL_FILTRATION_BACKWASH     "Rétro-lavage"
#define D_NEOPOOL_FILTRATION_NONE         ""                  // Filtration speed level
#define D_NEOPOOL_FILTRATION_SLOW         "lent"
#define D_NEOPOOL_FILTRATION_MEDIUM       "moyen"
#define D_NEOPOOL_FILTRATION_FAST         "rapide"
#define D_NEOPOOL_TYPE                    "Type"              // Sensor & relais names
#define D_NEOPOOL_REDOX                   "Redox"
#define D_NEOPOOL_CHLORINE                "Chlore"
#define D_NEOPOOL_CONDUCTIVITY            "Conductivité"
#define D_NEOPOOL_IONIZATION              "Ionisation"
#define D_NEOPOOL_HYDROLYSIS              "Hydrolyse"
#define D_NEOPOOL_RELAY                   "Relais"
#define D_NEOPOOL_RELAY_FILTRATION        "Filtration"        // Relay assignment
#define D_NEOPOOL_RELAY_LIGHT             "Lumière"
#define D_NEOPOOL_RELAY_PH_ACID           "Pompe acide"
#define D_NEOPOOL_RELAY_PH_BASE           "Pompe base"
#define D_NEOPOOL_RELAY_RX                "Pompe Redox"
#define D_NEOPOOL_RELAY_CL                "Pompe Chlore"
#define D_NEOPOOL_RELAY_CD                "Conductivité"
#define D_NEOPOOL_RELAY_HEATING           "Chauffage"
#define D_NEOPOOL_RELAY_UV                "UV"
#define D_NEOPOOL_RELAY_VALVE             "Valve"
#define D_NEOPOOL_RELAY_AUX               "Aux"
#define D_NEOPOOL_TIME                    "Durée"
#define D_NEOPOOL_FILT_MODE               "Filtration mode"
#define D_NEOPOOL_CELL_RUNTIME            "Utilisation cellule"
#define D_NEOPOOL_POLARIZATION            "Pol"               // Sensor status
#define D_NEOPOOL_PR_OFF                  "PrOff"
#define D_NEOPOOL_SETPOINT_OK             "OK"
#define D_NEOPOOL_COVER                   "Couverture"
#define D_NEOPOOL_SHOCK                   "Choc chlore"
#define D_NEOPOOL_STATUS_ON               "ON"
#define D_NEOPOOL_STATUS_OFF              "OFF"
#define D_NEOPOOL_STATUS_WAIT             "Attente"
#define D_NEOPOOL_STATUS_TANK             "Réservoir"
#define D_NEOPOOL_STATUS_FLOW             "Flux"
#define D_NEOPOOL_LOW                     "Bas"
#define D_NEOPOOL_FLOW1                   "FL1"
#define D_NEOPOOL_FLOW2                   "FL2"
#define D_NEOPOOL_PH_HIGH                 "Trop haut"          // ph Alarms
#define D_NEOPOOL_PH_LOW                  "Trop bas"
#define D_NEOPOOL_PUMP_TIME_EXCEEDED      "durée pompage expirée"

// xsns_106_gdk101.ino
#define D_AVG_RAD_DOSE                    "dose moyenne de rayonnement"
#define D_UNIT_US_H                       "µSv/h"

#endif  // _LANGUAGE_FR_FR_H_
